@@ -1,11 +1,14 @@
 # --------------------------------------------------------------- Imports ---------------------------------------------------------------- #
 
 # System
-from typing import Optional, Union, List, Dict, Any
+from typing import Optional, Union, List, Dict, Any, Union
 import os
 
 # Local
 from .category import Category
+from .sort_type import SortType
+from .product import *
+from .review import Review
 
 # ---------------------------------------------------------------------------------------------------------------------------------------- #
 
@@ -22,25 +25,31 @@ class AmazonBuddy:
         cls,
         search_term: str,
         category: Category = Category.ALL_DEPARTMENTS,
+        sort_type: Optional[SortType] = None,
         ignored_title_strs: List[str] = [],
         ignored_asins: List[str] = [],
         min_price: float = 50.0,
         min_reviews: int = 5,
         min_rating: float = 0.0,
         max_results: int = 100,
+        user_agent: Optional[str] = None,
         random_ua: bool = True,
-        sort: bool = True
-    ) -> Optional[List[str]]:
+        sort: bool = True,
+        debug: bool = False
+    ) -> Optional[List[Union[Dict, Product]]]:
         products = cls.__exec_cmd(
             'products',
+            user_agent=user_agent,
             random_ua=random_ua,
             extra_params={
                 '-k': '\'' + search_term + '\'',
                 '-c': category.value,
-                '-n': max_results,
+                '-n': max_results if max_results <= 500 else 500,
+                '--productsorttype': sort_type.value if sort_type else None,
                 '--min-rating': min_rating,
                 '--sort': sort
-            }
+            },
+            debug=debug
         )
 
         if products is None:
@@ -52,17 +61,16 @@ class AmazonBuddy:
             try:
                 from kcu import strings
 
-                product['price'] = float(product['price'].replace(',', ''))
-                product['asin'] = strings.between(product['url'].replace('%2F', '/'), '/dp/', '/')
+                product = Product(product)
 
                 if (
-                    product['price'] < min_price
+                    product.price.current < min_price
                     or
-                    product['reviews'] < min_reviews
+                    product.rating.count < min_reviews
                     or
-                    cls.__contains(product['asin'], ignored_asins)
+                    cls.__contains(product.asin, ignored_asins)
                     or
-                    cls.__contains_in(product['title'], ignored_title_strs)
+                    cls.__contains_in(product.title, ignored_title_strs)
                 ):
                     continue
 
@@ -71,35 +79,48 @@ class AmazonBuddy:
                 pass
 
         return filtered_products
-    
+
     @classmethod
     def get_reviews(
         cls,
         asin: str,
         min_rating: float = 0.0,
         max_results: int = 100,
+        user_agent: Optional[str] = None,
         random_ua: bool = True,
-        sort: bool = True
-    ) -> Optional[List[str]]:
-        return cls.__exec_cmd(
+        sort: bool = True,
+        debug: bool = False
+    ) -> Optional[List[Review]]:
+        js = cls.__exec_cmd(
             'reviews ' + asin,
+            user_agent=user_agent,
             random_ua=random_ua,
             extra_params={
                 '-n': max_results,
                 '--min-rating': min_rating,
                 '--sort': sort
-            }
+            },
+            debug=debug
         )
 
+        if js:
+            return [Review(j) for j in js]
+
+        return None
+
     @classmethod
     def get_product_details(
         cls,
         asin: str,
-        random_ua: bool = True
-    ) -> Optional[List[str]]:
+        user_agent: Optional[str] = None,
+        random_ua: bool = True,
+        debug: bool = False
+    ) -> Optional[Dict]:
         return cls.__exec_cmd(
             'asin ' + asin,
-            random_ua=random_ua
+            user_agent=user_agent,
+            random_ua=random_ua,
+            debug=debug
         )
 
 
@@ -109,18 +130,27 @@ class AmazonBuddy:
     def __exec_cmd(
         cls,
         feature: str,
+        user_agent: Optional[str] = None,
         random_ua: bool = True,
-        extra_params: Optional[Dict[str, Any]] = None
+        extra_params: Optional[Dict[str, Any]] = None,
+        debug: bool = False
     ) -> Optional[Union[List, Dict]]:
         params = extra_params or {}
-        params['--random-ua'] = random_ua
+
+        if user_agent:
+            params['--ua'] = user_agent
+        elif random_ua:
+            from fake_useragent import FakeUserAgent
+
+            params['--ua'] = FakeUserAgent().random
+
         params['--filetype'] = 'json'
 
         from kcu import sh, kjson, kpath
 
         resp = sh.sh(
             cls.__create_cmd(feature, params),
-            debug=True
+            debug=debug
         ).strip()
 
         try:
@@ -137,7 +167,7 @@ class AmazonBuddy:
             if path is None:
                 return None
 
-            print('path', path)
+            # print('path', path)
         except:
             return None
 
@@ -154,12 +184,11 @@ class AmazonBuddy:
         cmd = 'amazon-buddy' + ' ' + feature
 
         for k, v in params.items():
-            if k == '--random-ua':
-                k = '--ua'
+            if not v:
+                continue
 
-                from fake_useragent import FakeUserAgent
-
-                v = '\'' + FakeUserAgent().random.replace('\'', '\\\'') + '\''
+            if k == '--ua':
+                v = '\'' + v.replace('\'', '\\\'') + '\''
             if isinstance(v, bool):
                 v = str(v).lower()
 
